/**
* Klasse FreightEntry
* SchiffsFracht Dispositionsprogramm
* Frachtpapier, beinhaltet Freight Objekt
*/

import java.io.*;
import java.security.*;


public class FreightEntry implements Serializable
{
	// Status
	public static final int	INIT = 0;
	public static final int SGNREQUEST = 1;
	public static final int SIGNED = 2;
	public static final int GRANTED = 3;
	public static final int ANNOUNCED = 4;
	public static final int SET = 5;
	public static final int RELEASED = 6;

	private static final String stateStr[] = {"initialized", "sgn requested", "signed", "granted", "announced", "set", "released"};

	private Freight		freight;
	private SignedObject	signedFreight;
	private int		state;
	private int		signer;
	private int		creator;
	private int		frnr;

	public FreightEntry(int creator, Freight freight)
	{
		this.freight = freight;
		this.state = FreightEntry.INIT;
		this.signer = -1;
		this.signedFreight = null;
		this.frnr = creator * 1000 + freight.hashCode() % 1000;
	}

	public Freight getFreight()
	{
		return(this.freight);
	}

	public int getFreightNum()
	{
		return(this.frnr);
	}

	public int raiseState()
	{
		return(++this.state);
	}

	public int getState()
	{
		return(this.state);
	}

	public String getStateStr()
	{
		return(stateStr[this.state]);
	}

	public SignedObject getSignedObject()
	{
		return(this.signedFreight);
	}

	/** Erstellt eine digitale Signatur fuer Freight Objekt */
	public boolean sign(int signer, PrivateKey sigKey)
	{
		if (this.state != SGNREQUEST)
			return(false);

		try
		{
			this.signedFreight = new SignedObject(this.freight, sigKey, Signature.getInstance("DSA"));
			this.state = FreightEntry.SIGNED;
		}
		catch (Exception e)
		{
			return(false);
		}
		return(true);
	}

	public boolean verify(PublicKey validKey)
	{
		try
		{
			return(this.signedFreight.verify(validKey, Signature.getInstance("DSA")));
		}
		catch (InvalidKeyException e) {}
		catch (NoSuchAlgorithmException e) {}
		catch (SignatureException e) {}

		return(false);
	}
}
