/**
* Klasse SFRegistryImpl
* SchiffsFracht Dispositionsprogramm
* Registriert, updated Servers und
* ueberprueft, ob ein Server noch lebt
*/

import java.util.*;
import java.lang.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;


public class SFRegistryImpl extends UnicastRemoteObject implements SFRegistry
{
	private ServerMap	serverMap;
	private Date		date;


	public SFRegistryImpl(ServerMap map) throws RemoteException
	{
		super();
		this.serverMap = map;
		this.date = new Date();
	}

	/** Server ruf hello und registriert sich in ServerMap
	  * Rueckgabe: Servernummer
	  */
	public synchronized int hello(ServerInfo s) throws RemoteException
	{
		int	n;

		this.logConsole("update received from " + s.getHostname());
		if (!serverMap.exists(s))
		{
			n = this.updateComplete(s);	//Updated s mit alle infos
			if (n < 0)
				this.logConsole("reverse update not possible, update from " + s.getHostname() + " ignored");
			else
			{
				n = this.updateServers(s, SFServer.ADD);	//alle Servers mit s updaten
				this.logConsole(n + " servers updated");
				serverMap.put(s);					//s in serverMap hinzufuegen
			}
		}
		else
			this.logConsole(s.getHostname() + " already exists, update ignored.");

		return(s.getSrvNum());
	}


	public boolean verify(ServerInfo verify, ServerInfo calling)
	{
		return(true);
	}

	/** Ausgabe ... */
	private void logConsole(String s)
	{
		/* Datum auf aktuelle Zeit setzen. */
		this.date.setTime(System.currentTimeMillis());
		System.out.println(this.date + " " + s);
	}

	/** Alle in servers in serverMap updaten */
	private int updateServers(ServerInfo s, int m)
	{
		SFServer	sInt;
		ServerInfo	sInfo;
		String		rmiString;
		int		n = 0;

		Iterator i = this.serverMap.values().iterator();
		while (i.hasNext())
		{
			sInfo = (ServerInfo) i.next();
			rmiString = serverMap.getRmiString(sInfo);

			if ((sInt = this.lookup(rmiString)) == null)
			{
				this.logConsole("cannot lookup " + rmiString);
				continue;
			}

			if (!this.update(sInt, s, m))
			{
				this.logConsole("cannot update date for " + rmiString);
				continue;
			}

			n++;
		}
		return(n);
	}

	/** Updated s mit alle infos von serverMap */
	private int updateComplete(ServerInfo s)
	{
		SFServer	sInt;
		int		n = 0;

		if ((sInt = this.lookup(serverMap.getRmiString(s))) == null)
		{
			this.logConsole("cannot lookup calling server " + serverMap.getRmiString(s));
			return(-1);
		}

		Iterator i = this.serverMap.values().iterator();
		while (i.hasNext())
		{
			if (this.update(sInt, (ServerInfo) i.next(), SFServer.ADD))
				n++;
		}
		return(n);
	}


	private boolean update(SFServer sInt, ServerInfo sInfo, int m)
	{
		try
		{
			sInt.update(sInfo, m);
		}
		catch (Exception e)
		{
			return(false);
		}
		return(true);
	}


	private SFServer lookup(String s)
	{
		SFServer	si;
		try
		{
			si = (SFServer) Naming.lookup(s);
		}
		catch (Exception e)
		{
			si = null;
		}

		return(si);
	}

	public static void main(String[] argv)
	{
		ServerMap	serverMap = new ServerMap();
		CommandLineInterface	cli;
		SFRegistryImpl	obj = null;
		String		server = "localhost";

		// use a Security Manager
		if (System.getSecurityManager() == null)
			System.setSecurityManager(new RMISecurityManager());

		try
		{
			/* Hostname des Rechners holen. */
			server = InetAddress.getLocalHost().getHostName();

			/* MessageServer RMI-Object instantiieren. */
			obj = new SFRegistryImpl(serverMap);

			/* MessageServer RMI-Object in der RMI-Registry
			 * registrieren. */
			Naming.bind("rmi://" + server + "/" + SFRegistry.rmiName, obj);
		}
		catch (Exception e)
		{
			/* Fehlermeldung auf Console ausgeben ... */
			System.err.println("*** ERROR: " + e.getMessage());
			/* ... und Server beenden. */
			System.exit(1);
		}

		/* CommandLineInterface erstellen. */
		cli = new CommandLineInterface(SFRegistry.rmiName + "@" + server + "# ");
		cli.registerCommand("servers", serverMap);
		/* Information auf der Console ausgeben. */
		obj.logConsole("\"rmi://" + server + "/" + SFRegistry.rmiName + "\" bound in RMIregistry");
		obj.logConsole("Server ready, starting Console");
		/* CommandLineInterface starten. */
		cli.startCLI();

		try
		{
			Naming.unbind("rmi://" + server + "/" + SFRegistry.rmiName);
		}
		catch (Exception e) {}

		System.exit(0);
	}
}
