/**
* Abstrakte Klasse SFServerImpl
* SchiffsFracht Dispositionsprogramm
*/

import java.util.*;
import java.lang.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.security.*;


public abstract class SFServerImpl extends UnicastRemoteObject implements SFServer
{
	/** Hostname des defaultmssigen Registry Servers. */
	private static HashMap	sfInstance = new HashMap();

	protected ServerMap	serverMap;
	protected FreightMap	freightMap;
	protected ServerInfo	sInfo;
	protected KeyPair	keyPair;

	private KeyPairGenerator	kGen;
	private Date		date;

	/** protected.. alles geht ueber getInstace() */
	protected SFServerImpl(String host, String desc, int type) throws RemoteException, AlreadyBoundException
	{
		super();

		this.serverMap = new ServerMap();
		this.freightMap = new FreightMap();
		this.date = new Date();

		try
		{
			kGen = KeyPairGenerator.getInstance("DSA");
		}
		catch (NoSuchAlgorithmException e)
		{
			System.err.println("FATAL ERROR: " + e.getMessage());
			System.exit(1);
		}

		kGen.initialize(1024);
		this.keyPair = kGen.genKeyPair();

		this.sInfo = new ServerInfo(host, desc, type, this.keyPair.getPublic());

		try
		{
			Naming.bind(ServerMap.getRmiString(this.sInfo), this);
		}
		catch (MalformedURLException e)
		{
			System.err.println("FATAL ERROR: " + e.getMessage());
			System.exit(1);
		}

		this.logConsole(ServerMap.getRmiString(this.sInfo) + " bound in RMIregistry.");

	}

	// instantiiert und registriert obj bei SF-Registry
	public static SFServerImpl getInstance(String host, String desc, int type)
	{
		SFServerImpl	si = null;
		String		implKey = host + ":" + desc + ":" + type;


		if (sfInstance.containsKey(implKey))
			return((SFServerImpl) sfInstance.get(implKey));

		try
		{
			switch (type)
			{
				case ServerInfo.CARRIER :
					si = new CarrierImpl(host, desc);
					break;
				case ServerInfo.PORT :
					si = new PortImpl(host, desc);
					break;
				case ServerInfo.CUSTOMS :
					si = new CustomsImpl(host, desc);
					break;
				default :
					return(null);
			}

			si.register(Config.getInstance().get("registry").toString());
		}
		catch (RemoteException e)
		{
			System.err.println("ERROR: " + e.getMessage());
			return(null);
		}
		catch (AlreadyBoundException e)
		{
			System.err.println("ERROR: " + e.getMessage());
			return(null);
		}
		catch (NotBoundException e)
		{
			System.err.println("ERROR: " + e.getMessage());
			return(null);
		}
		sfInstance.put(implKey, si);
		return(si);
	}


	public static Iterator iterator()
	{
		return(sfInstance.values().iterator());
	}


	public static SFServerImpl getInstanceKey(String implKey)
	{
		return((SFServerImpl) sfInstance.get(implKey));
	}


	private boolean register(String server) throws RemoteException, NotBoundException
	{
		SFRegistry	rInt;
		String		rmiString;

		if (server == null)
			return(false);

		rmiString = "rmi://" + server + "/" + SFRegistry.rmiName;
		this.logConsole("looking up \"" + rmiString + "\"");

		try
		{
			rInt = (SFRegistry) Naming.lookup(rmiString);
		}
		catch (MalformedURLException e)
		{
			this.logConsole("cannot lookup " + rmiString + ", malformed URL");
			return(false);
		}

		rInt.hello(sInfo);

		return(true);
	}


	public void unregister()
	{
		try
		{
			Naming.unbind(ServerMap.getRmiString(this.sInfo));
		}
		catch (Exception e) {}
	}

	public ServerInfo getServerInfo()
	{
		return(this.sInfo);
	}

	public ServerMap getServerMap()
	{
		return(this.serverMap);
	}

	public FreightMap getFreightMap()
	{
		return(this.freightMap);
	}

	public void logConsole(String s)
	{
		/* Datum auf aktuelle Zeit setzen. */
		this.date.setTime(System.currentTimeMillis());
		System.out.println(this.date + " " + s);
	}


	protected synchronized boolean testAndRaise(int state, FreightEntry entry)
	{
		/* Falls Frachtpapier existiert -> false zurck */
		if (this.freightMap.exists(entry))
			return(false);

		/* Falls Frachtpapier nicht SIGNED -> false zurck */
		if (entry.getState() != state)
			return(false);

		/* Status hinaufsetzen und in Map aufnehmen. */
		entry.raiseState();
		this.freightMap.put(entry);

		return(true);
	}

	public abstract void initCmd(CommandLineInterface cli);

	public synchronized void update(ServerInfo s, int mode) throws RemoteException
	{
		switch (mode)
		{
			case SFServer.ADD :
				serverMap.put(s);
				this.logConsole("update received: ADD " + s.getHostname() + " (" + s.getTypeStr() + ":" + s.getDescription() + ")");
				break;
			case SFServer.DEL :
				serverMap.remove(s);
				this.logConsole("update received: DEL " + s.getHostname());
				break;
			default :
				this.logConsole("illegal update mode selected, update ignored");
		}
	}


	public boolean isAlive() throws RemoteException
	{
		return(true);
	}
}
