import java.util.*;
import java.rmi.*;
import java.net.*;
import java.io.*;

public class CmdRequestPapers implements Command
{
	private final int	NUMPARAMS = 2;
	private final String	USAGE = "<freight#> <server#>\n request papers for <freight#> at <server#>.";
	private SFServerImpl	server;

	public CmdRequestPapers(SFServerImpl server)
	{
		this.server = server;
	}

	public int execute(PrintWriter out, String cmdString, String paramString)
	{
		Customs		customs = null;
		FreightEntry	freight;
		ServerInfo	destInfo;
		String		p[] = new String[NUMPARAMS];
		int		f = 0, s = 0;
		int		tcount = 0;
		StringTokenizer	st = new StringTokenizer(paramString, " ");

		while (st.hasMoreTokens() && (tcount < NUMPARAMS))
			p[tcount++] = st.nextToken();

		if ((tcount != NUMPARAMS) || (p[0].equals("?")))
		{
			System.out.println(cmdString + " " + USAGE);
			return(Command.OK);
		}

		try
		{
			f = Integer.parseInt(p[0]);
			s = Integer.parseInt(p[1]);
		}
		catch (NumberFormatException e)
		{
			return(Command.SYNTAXERROR);
		}

		if ((freight = server.getFreightMap().get(f)) == null)
		{
			System.out.println("*** FreightEntry #" + f + " does not exist.");
			return(Command.OK);
		}

		if (freight.getState() != FreightEntry.INIT)
		{
			System.out.println("*** FreightEntry #" + f + " is not in inital state.");
			return(Command.OK);
		}

		if ((destInfo = server.getServerMap().get(s)) == null)
		{
			System.out.println("*** Server #" + s + " does not exist.");
			return(Command.OK);
		}
		
		if (destInfo.getType() != ServerInfo.CUSTOMS)
		{
			System.out.println("*** Server #" + s + " is not CUSTOMS server.");
			return(Command.OK);
		}

		try
		{
//			SFServerImpl si = (SFServerImpl) Naming.lookup(ServerMap.getRmiString(destInfo));
			customs = (Customs) Naming.lookup(ServerMap.getRmiString(destInfo));
		}
		catch (MalformedURLException e)
		{
			server.logConsole("cannot lookup " + ServerMap.getRmiString(destInfo) + ": " + e.getMessage());
			return(Command.FINERROR);
		}
		catch (NotBoundException e)
		{
			server.logConsole("cannot lookup " + ServerMap.getRmiString(destInfo) + ": " + e.getMessage());
			return(Command.FINERROR);
		}
		catch (RemoteException e)
		{
			server.logConsole("cannot lookup " + ServerMap.getRmiString(destInfo) + ": " + e.getMessage());
			return(Command.FINERROR);
		}

		try
		{
			if (!customs.requestPapers(freight))
			{
				System.out.println("request not successful, request was previously done.");
				return(Command.OK);
			}
		}
		catch (RemoteException e)
		{
			System.out.println("communication error during request");
			return(Command.FINERROR);
		}

		server.getFreightMap().remove(freight);

		return(Command.OK);
	}
}
