import java.util.*;
import java.rmi.*;
import java.net.*;
import java.io.*;

public class CmdSetArrival implements Command
{
	private final String	USAGE = "<freight#> <server#>\n set arrival of <freight#> at port and send papers to customs <server#>.";
	private SFServerImpl	server;

	public CmdSetArrival(SFServerImpl server)
	{
		this.server = server;
	}

	public int execute(PrintWriter out, String cmdString, String paramString)
	{
		Customs customs = null;
		FreightEntry	freight;
		ServerInfo	destInfo;
		String[] param;
		int f, s, i;

		StringTokenizer	st = new StringTokenizer(paramString, " ");

		param = new String[st.countTokens()];
		for (i=0; i<param.length; i++) {
			param[i] = st.nextToken();
		}

		if ((param.length != 2))
		{
			out.println(cmdString + " " + USAGE);
			return(Command.OK);
		}

		try
		{
			f = Integer.parseInt(param[0]);
			s = Integer.parseInt(param[1]);
		}

		catch (NumberFormatException e)
		{
			return(Command.SYNTAXERROR);
		}

		if ((freight = server.getFreightMap().get(f)) == null)
		{
			out.println("*** FreightEntry #" + f + " does not exist.");
			return(Command.OK);
		}

		if (freight.getState() != FreightEntry.ANNOUNCED)
		{
			out.println("*** FreightEntry #" + f + " is not grated.");
			return(Command.OK);
		}

		if ((destInfo = server.getServerMap().get(s)) == null)
		{
			out.println("*** Server #" + s + " does not exist.");
			return(Command.OK);
		}

		if (destInfo.getType() != ServerInfo.CUSTOMS)
		{
			out.println("*** Server #" + s + " is not CUSTOMS server.");
			return(Command.OK);
		}

		try
		{
			customs = (Customs) Naming.lookup(ServerMap.getRmiString(destInfo));
		}

		catch (MalformedURLException e)
		{
			server.logConsole("cannot lookup " + ServerMap.getRmiString(destInfo) + ": " + e.getMessage());
			return(Command.FINERROR);
		}
		catch (NotBoundException e)
		{
			server.logConsole("cannot lookup " + ServerMap.getRmiString(destInfo) + ": " + e.getMessage());
			return(Command.FINERROR);
		}
		catch (RemoteException e)
		{
			server.logConsole("cannot lookup " + ServerMap.getRmiString(destInfo) + ": " + e.getMessage());
			return(Command.FINERROR);
		}

		try
		{
			if (!customs.setArrival(freight))
			{
				out.println("request not successful, request was previously done.");
				return(Command.OK);
			}
		}
		catch (RemoteException e)
		{
			out.println("communication error during request");
			return(Command.FINERROR);
		}

		server.getFreightMap().remove(freight);

		return(Command.OK);
	}
}
